"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var gesturehandler_common_1 = require("./gesturehandler.common");
exports.GestureHandlerStateEvent = gesturehandler_common_1.GestureHandlerStateEvent;
exports.GestureHandlerTouchEvent = gesturehandler_common_1.GestureHandlerTouchEvent;
exports.GestureState = gesturehandler_common_1.GestureState;
exports.HandlerType = gesturehandler_common_1.HandlerType;
exports.ViewDisposeEvent = gesturehandler_common_1.ViewDisposeEvent;
exports.ViewInitEvent = gesturehandler_common_1.ViewInitEvent;
var utils_1 = require("tns-core-modules/utils/utils");
var application_1 = require("tns-core-modules/application/application");
var page_1 = require("tns-core-modules/ui/page/page");
function initRootViewGestureHandler() {
    if (exports.RootViewGestureHandler) {
        return;
    }
    var RootViewGestureHandlerImpl = /** @class */ (function (_super) {
        __extends(RootViewGestureHandlerImpl, _super);
        function RootViewGestureHandlerImpl() {
            var _this = _super.call(this) || this;
            return global.__native(_this);
        }
        RootViewGestureHandlerImpl.prototype.getView = function () {
            return _super.prototype.getView.call(this);
        };
        RootViewGestureHandlerImpl.prototype.onHandle = function (event) {
            var currentState = this.getState();
            if (currentState === com.swmansion.gesturehandler.GestureHandler.STATE_UNDETERMINED) {
                this.begin();
                this.getView().setShouldIntercept(true);
            }
            if (event.getActionMasked() === android.view.MotionEvent.ACTION_UP) {
                this.end();
            }
        };
        RootViewGestureHandlerImpl.prototype.onCancel = function () {
            this.getView().setShouldIntercept(false);
            var time = android.os.SystemClock.uptimeMillis();
            var event = android.view.MotionEvent.obtain(time, time, android.view.MotionEvent.ACTION_CANCEL, 0, 0, 0);
            event.setAction(android.view.MotionEvent.ACTION_CANCEL);
        };
        return RootViewGestureHandlerImpl;
    }(com.swmansion.gesturehandler.GestureHandler));
    exports.RootViewGestureHandler = RootViewGestureHandlerImpl;
}
function initPageLayout() {
    if (exports.PageLayout) {
        return;
    }
    var PageLayoutImpl = /** @class */ (function (_super) {
        __extends(PageLayoutImpl, _super);
        function PageLayoutImpl(context) {
            var _this = _super.call(this, context) || this;
            _this.mShouldIntercept = false;
            _this.mPassingTouch = false;
            return global.__native(_this);
        }
        PageLayoutImpl.prototype.setShouldIntercept = function (value) {
            this.mShouldIntercept = value;
        };
        PageLayoutImpl.prototype.setPassingTouch = function (value) {
            this.mPassingTouch = value;
        };
        Object.defineProperty(PageLayoutImpl.prototype, "registry", {
            get: function () {
                return this.mRegistry;
            },
            enumerable: true,
            configurable: true
        });
        // requestDisallowInterceptTouchEvent(disallowIntercept) {
        //     console.log('requestDisallowInterceptTouchEvent');
        //     if (this.mGestureRootHelper != null) {
        //         this.mGestureRootHelper.requestDisallowInterceptTouchEvent(disallowIntercept);
        //     }
        //     super.requestDisallowInterceptTouchEvent(disallowIntercept);
        // }
        // dispatchTouchEvent(ev) {
        //     if (this.mGestureRootHelper != null && this.mGestureRootHelper.dispatchTouchEvent(ev)) {
        //         return true;
        //     }
        //     return super.dispatchTouchEvent(ev);
        // }
        PageLayoutImpl.prototype.tryCancelAllHandlers = function () {
            // In order to cancel handlers we activate handler that is hooked to the root view
            if (this.rootGestureHandler != null && this.rootGestureHandler.getState() === com.swmansion.gesturehandler.GestureHandler.STATE_BEGAN) {
                // Try activate main JS handler
                this.rootGestureHandler.activate();
                this.rootGestureHandler.end();
            }
        };
        PageLayoutImpl.prototype.requestDisallowInterceptTouchEvent = function (disallowIntercept) {
            // console.log('requestDisallowInterceptTouchEvent', disallowIntercept);
            // If this method gets called it means that some native view is attempting to grab lock for
            // touch event delivery. In that case we cancel all gesture recognizers
            if (this.mOrchestrator != null && !this.mPassingTouch) {
                // if we are in the process of delivering touch events via GH orchestrator, we don't want to
                // treat it as a native gesture capturing the lock
                this.tryCancelAllHandlers();
            }
            _super.prototype.requestDisallowInterceptTouchEvent.call(this, disallowIntercept);
        };
        PageLayoutImpl.prototype.dispatchTouchEventToOrchestrator = function (ev) {
            // console.log('dispatchTouchEventToOrchestrator', ev, this.mShouldIntercept);
            this.mPassingTouch = true;
            this.mOrchestrator.onTouchEvent(ev);
            this.mPassingTouch = false;
            return this.mShouldIntercept;
        };
        PageLayoutImpl.prototype.dispatchTouchEvent = function (ev) {
            if (this.dispatchTouchEventToOrchestrator(ev)) {
                return true;
            }
            var handled = _super.prototype.dispatchTouchEvent.call(this, ev);
            // console.log('dispatchTouchEvent', ev, handled, this.mShouldIntercept);
            // we need to always return true or gestures wont work on layouts because they don't handle touch so dispatchTouchEvent returns false
            return true;
        };
        // onInterceptTouchEvent(ev: android.view.MotionEvent) {
        //     return this.mShouldIntercept;
        // }
        // onTouchEvent(ev: android.view.MotionEvent) {
        //     console.log('onTouchEvent', ev);
        //     this.mOrchestrator.onTouchEvent(ev);
        //     return super.onTouchEvent(ev);
        // }
        /**
         * This method is used to enable root view to start processing touch events through the gesture
         * handler library logic. Unless this method is called (which happens as a result of instantiating
         * new gesture handler from JS) the root view component will just proxy all touch related methods
         * to its superclass. Thus in the "disabled" state all touch related events will fallback to
         * default RN behavior.
         */
        PageLayoutImpl.prototype.initialize = function () {
            this.mRegistry = new com.swmansion.gesturehandler.GestureHandlerRegistryImpl();
            this.configurationHelper = new com.swmansion.gesturehandler.ViewConfigurationHelper({
                getPointerEventsConfigForView: function (view) {
                    return view.isEnabled() ? com.swmansion.gesturehandler.PointerEventsConfig.AUTO : com.swmansion.gesturehandler.PointerEventsConfig.NONE;
                },
                isViewClippingChildren: function (parent) {
                    return false;
                },
                getChildInDrawingOrderAtIndex: function (parent, index) {
                    return parent.getChildAt(index);
                }
            });
            this.mOrchestrator = new com.swmansion.gesturehandler.GestureHandlerOrchestrator(this, this.mRegistry, this.configurationHelper);
            console.log(this.constructor.name, 'initialize', this.mOrchestrator, this.mRegistry);
            this.mOrchestrator.setMinimumAlphaForTraversal(0.01);
            initRootViewGestureHandler();
            var tag = -12345;
            this.rootGestureHandler = new exports.RootViewGestureHandler();
            this.rootGestureHandler.setTag(tag);
            this.mRegistry.registerHandler(this.rootGestureHandler);
            // registry.attachHandlerToView(this.rootGestureHandler.getTag(), this);
        };
        PageLayoutImpl.prototype.tearDown = function () {
            console.log(this.constructor.name, 'tearDown', this.mOrchestrator, this.mRegistry);
            this.configurationHelper = null;
            this.mOrchestrator = null;
            this.mRegistry = null;
            // if (this.mGestureRootHelper != null) {
            //     this.mGestureRootHelper.tearDown();
            //     this.mGestureRootHelper = null;
            // }
        };
        return PageLayoutImpl;
    }(org.nativescript.widgets.GridLayout));
    exports.PageLayout = PageLayoutImpl;
}
var PageGestureExtended = /** @class */ (function (_super) {
    __extends(PageGestureExtended, _super);
    function PageGestureExtended() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PageGestureExtended.prototype.initNativeView = function () {
        this.nativeView.initialize();
    };
    PageGestureExtended.prototype.disposeNativeView = function () {
        this.nativeView.tearDown();
    };
    Object.defineProperty(PageGestureExtended.prototype, "registry", {
        get: function () {
            return this.nativeView && this.nativeView.registry;
        },
        enumerable: true,
        configurable: true
    });
    return PageGestureExtended;
}(page_1.Page));
function install() {
    gesturehandler_common_1.install();
    var NSPage = require('tns-core-modules/ui/page/page').Page;
    NSPage.prototype.createNativeView = function () {
        initPageLayout();
        var layout = new exports.PageLayout(this._context);
        this.gestureRegistry = layout.registry;
        layout.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.auto));
        layout.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.star));
        return layout;
    };
    gesturehandler_common_1.applyMixins(NSPage, [PageGestureExtended]);
}
exports.install = install;
var KEY_HIT_SLOP_LEFT = 'left';
var KEY_HIT_SLOP_TOP = 'top';
var KEY_HIT_SLOP_RIGHT = 'right';
var KEY_HIT_SLOP_BOTTOM = 'bottom';
var KEY_HIT_SLOP_VERTICAL = 'vertical';
var KEY_HIT_SLOP_HORIZONTAL = 'horizontal';
var KEY_HIT_SLOP_WIDTH = 'width';
var KEY_HIT_SLOP_HEIGHT = 'height';
var Handler = /** @class */ (function (_super) {
    __extends(Handler, _super);
    function Handler() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.tag = 0;
        return _this;
    }
    Handler.prototype.getExtraData = function (handler) {
        return {
            numberOfPointers: handler.getNumberOfPointers()
        };
    };
    Handler.prototype.initNativeView = function (native, options) {
        var _this = this;
        _super.prototype.initNativeView.call(this, native, options);
        this.native.setTag(this.tag);
        this.touchListener = new com.swmansion.gesturehandler.OnTouchEventListener({
            onTouchEvent: function (handler, event) {
                // console.log('onTouchEvent', handler, event.getAction());
                var view = handler.getView();
                _this.notify({
                    eventName: gesturehandler_common_1.GestureHandlerTouchEvent,
                    object: _this,
                    data: {
                        state: handler.getState(),
                        android: view,
                        extraData: _this.getExtraData(handler),
                        view: view.nsView ? view.nsView.get() : null
                    }
                });
            },
            onStateChange: function (handler, state, prevState) {
                var view = handler.getView();
                _this.notify({
                    eventName: gesturehandler_common_1.GestureHandlerStateEvent,
                    object: _this,
                    data: {
                        state: state,
                        prevState: prevState,
                        android: view,
                        extraData: _this.getExtraData(handler),
                        view: view.nsView ? view.nsView.get() : null
                    }
                });
            }
        });
        native.setOnTouchEventListener(this.touchListener);
        this.manager.get().configureInteractions(this, options);
    };
    Handler.prototype.disposeNativeView = function () {
        this.native.setInteractionController(null);
        this.native.setOnTouchEventListener(null);
        this.touchListener = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Handler.prototype.onTouchEvent = function (handler, event) { };
    Handler.prototype.onStateChange = function (handler, state, prevState) { };
    Handler.prototype.setTag = function (tag) {
        this.tag = tag;
        if (this.native) {
            this.native.setTag(tag);
        }
    };
    Handler.prototype.getTag = function () {
        return this.tag;
        // return this.getNative().getTag();
    };
    Handler.prototype.getView = function () {
        return this.getNative().getView();
    };
    Handler.prototype.cancel = function () {
        return this.getNative().cancel();
    };
    Handler.prototype.attachToView = function (view) {
        this.manager.get().attachGestureHandler(this, view);
    };
    Handler.prototype.detachFromView = function (view) {
        this.manager.get().detachGestureHandler(this, view);
    };
    __decorate([
        gesturehandler_common_1.nativeProperty({
            nativeSetterName: 'setHitSlop',
            nativegetterName: 'getHitSlop',
            converter: {
                toNative: function (value) {
                    var HIT_SLOP_NONE = com.swmansion.gesturehandler.GestureHandler.HIT_SLOP_NONE;
                    if (typeof value === 'number') {
                        var hitSlop = utils_1.layout.toDevicePixels(value);
                        return [hitSlop, hitSlop, hitSlop, hitSlop, HIT_SLOP_NONE, HIT_SLOP_NONE];
                    }
                    else {
                        var left = HIT_SLOP_NONE, top_1 = HIT_SLOP_NONE, right = HIT_SLOP_NONE, bottom = HIT_SLOP_NONE;
                        var width = HIT_SLOP_NONE, height = HIT_SLOP_NONE;
                        if (value.hasOwnProperty(KEY_HIT_SLOP_HORIZONTAL)) {
                            var horizontalPad = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_HORIZONTAL]);
                            left = right = horizontalPad;
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_VERTICAL)) {
                            var verticalPad = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_VERTICAL]);
                            top_1 = bottom = verticalPad;
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_LEFT)) {
                            left = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_LEFT]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_TOP)) {
                            top_1 = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_TOP]);
                        }
                        if (value.hasKey(KEY_HIT_SLOP_RIGHT)) {
                            right = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_RIGHT]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_BOTTOM)) {
                            bottom = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_BOTTOM]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_WIDTH)) {
                            width = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_WIDTH]);
                        }
                        if (value.hasOwnProperty(KEY_HIT_SLOP_HEIGHT)) {
                            height = utils_1.layout.toDevicePixels(value[KEY_HIT_SLOP_HEIGHT]);
                        }
                        return [left, top_1, right, bottom, width, height];
                    }
                }
            }
        }),
        __metadata("design:type", Object)
    ], Handler.prototype, "hitSlop", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "enabled", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "shouldCancelWhenOutside", void 0);
    return Handler;
}(gesturehandler_common_1.BaseNative));
exports.Handler = Handler;
var TapGestureHandler = /** @class */ (function (_super) {
    __extends(TapGestureHandler, _super);
    function TapGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TapGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.TapGestureHandler();
    };
    TapGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            x: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY())
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "numberOfTaps", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDurationMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDelayMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMaxDx' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDeltaX", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMaxDy' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDeltaY", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "maxDist", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setMinNumberOfPointers' }),
        __metadata("design:type", Number)
    ], TapGestureHandler.prototype, "minPointers", void 0);
    return TapGestureHandler;
}(Handler));
exports.TapGestureHandler = TapGestureHandler;
var PanGestureHandler = /** @class */ (function (_super) {
    __extends(PanGestureHandler, _super);
    function PanGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PanGestureHandler.prototype.createNative = function (options) {
        var context = application_1.android.context;
        return new com.swmansion.gesturehandler.PanGestureHandler(context);
    };
    PanGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            x: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: utils_1.layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: utils_1.layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY()),
            translationX: utils_1.layout.toDeviceIndependentPixels(handler.getTranslationX()),
            translationY: utils_1.layout.toDeviceIndependentPixels(handler.getTranslationY()),
            velocityX: utils_1.layout.toDeviceIndependentPixels(handler.getVelocityX()),
            velocityY: utils_1.layout.toDeviceIndependentPixels(handler.getVelocityY())
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minDist", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetXStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetXEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetXStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetXEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetYStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "activeOffsetYEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetYStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "failOffsetYEnd", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocity", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocityX", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minVelocityY", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "minPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "maxPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ nativeSetterName: 'setAverageTouches' }),
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "avgTouches", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], PanGestureHandler.prototype, "numberOfPointers", void 0);
    return PanGestureHandler;
}(Handler));
exports.PanGestureHandler = PanGestureHandler;
var PinchGestureHandler = /** @class */ (function (_super) {
    __extends(PinchGestureHandler, _super);
    function PinchGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PinchGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.PinchGestureHandler();
    };
    PinchGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            scale: handler.getScale(),
            focalX: utils_1.layout.toDeviceIndependentPixels(handler.getFocalPointX()),
            focalY: utils_1.layout.toDeviceIndependentPixels(handler.getFocalPointY()),
            velocity: handler.getVelocity()
        });
    };
    return PinchGestureHandler;
}(Handler));
exports.PinchGestureHandler = PinchGestureHandler;
var FlingGestureHandler = /** @class */ (function (_super) {
    __extends(FlingGestureHandler, _super);
    function FlingGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    FlingGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.FlingGestureHandler();
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], FlingGestureHandler.prototype, "numberOfPointers", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], FlingGestureHandler.prototype, "direction", void 0);
    return FlingGestureHandler;
}(Handler));
exports.FlingGestureHandler = FlingGestureHandler;
var LongPressGestureHandler = /** @class */ (function (_super) {
    __extends(LongPressGestureHandler, _super);
    function LongPressGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LongPressGestureHandler.prototype.createNative = function (options) {
        var context = application_1.android.context;
        return new com.swmansion.gesturehandler.LongPressGestureHandler(context);
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Number)
    ], LongPressGestureHandler.prototype, "minDurationMs", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty({ converter: { fromNative: utils_1.layout.toDevicePixels } }),
        __metadata("design:type", Number)
    ], LongPressGestureHandler.prototype, "maxDist", void 0);
    return LongPressGestureHandler;
}(Handler));
exports.LongPressGestureHandler = LongPressGestureHandler;
var RotationGestureHandler = /** @class */ (function (_super) {
    __extends(RotationGestureHandler, _super);
    function RotationGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RotationGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.RotationGestureHandler();
    };
    RotationGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            rotation: handler.getRotation(),
            anchorX: utils_1.layout.toDeviceIndependentPixels(handler.getAnchorX()),
            anchorY: utils_1.layout.toDeviceIndependentPixels(handler.getAnchorY()),
            velocity: handler.getVelocity()
        });
    };
    return RotationGestureHandler;
}(Handler));
exports.RotationGestureHandler = RotationGestureHandler;
var NativeViewGestureHandler = /** @class */ (function (_super) {
    __extends(NativeViewGestureHandler, _super);
    function NativeViewGestureHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NativeViewGestureHandler.prototype.createNative = function (options) {
        return new com.swmansion.gesturehandler.NativeViewGestureHandler();
    };
    NativeViewGestureHandler.prototype.getExtraData = function (handler) {
        return Object.assign(_super.prototype.getExtraData.call(this, handler), {
            pointerInside: handler.isWithinBounds()
        });
    };
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], NativeViewGestureHandler.prototype, "shouldActivateOnStart", void 0);
    __decorate([
        gesturehandler_common_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], NativeViewGestureHandler.prototype, "disallowInterruption", void 0);
    return NativeViewGestureHandler;
}(Handler));
exports.NativeViewGestureHandler = NativeViewGestureHandler;
function initGestureHandlerInteractionController() {
    if (exports.GestureHandlerInteractionController) {
        return;
    }
    var GestureHandlerInteractionControllerImpl = /** @class */ (function (_super) {
        __extends(GestureHandlerInteractionControllerImpl, _super);
        function GestureHandlerInteractionControllerImpl() {
            var _this = _super.call(this) || this;
            _this.mWaitForRelations = {};
            _this.mSimultaneousRelations = {};
            return global.__native(_this);
        }
        GestureHandlerInteractionControllerImpl.prototype.dropRelationsForHandlerWithTag = function (handlerTag) {
            delete this.mWaitForRelations[handlerTag];
            delete this.mSimultaneousRelations[handlerTag];
        };
        GestureHandlerInteractionControllerImpl.prototype.configureInteractions = function (handler, config) {
            handler.setInteractionController(this);
            if (config) {
                // console.log('configureInteractions', handler.getTag(), config);
                if (config.waitFor) {
                    this.mWaitForRelations[handler.getTag()] = config.waitFor;
                }
                if (config.simultaneousHandlers) {
                    this.mSimultaneousRelations[handler.getTag()] = config.simultaneousHandlers;
                }
            }
        };
        GestureHandlerInteractionControllerImpl.prototype.shouldWaitForHandlerFailure = function (handler, otherHandler) {
            var waitForTags = this.mWaitForRelations[handler.getTag()];
            console.log('shouldWaitForHandlerFailure', handler.getTag(), waitForTags);
            if (waitForTags != null) {
                for (var i = 0; i < waitForTags.length; i++) {
                    if (waitForTags[i] === otherHandler.getTag()) {
                        return true;
                    }
                }
            }
            return false;
        };
        GestureHandlerInteractionControllerImpl.prototype.shouldRequireHandlerToWaitForFailure = function (handler, otherHandler) {
            return false;
        };
        GestureHandlerInteractionControllerImpl.prototype.shouldHandlerBeCancelledBy = function (handler, otherHandler) {
            return false;
        };
        GestureHandlerInteractionControllerImpl.prototype.shouldRecognizeSimultaneously = function (handler, otherHandler) {
            var simultHandlerTags = this.mSimultaneousRelations[handler.getTag()];
            console.log('shouldRecognizeSimultaneously', handler.getTag(), simultHandlerTags);
            if (simultHandlerTags != null) {
                for (var i = 0; i < simultHandlerTags.length; i++) {
                    if (simultHandlerTags[i] === otherHandler.getTag()) {
                        return true;
                    }
                }
            }
            return false;
        };
        GestureHandlerInteractionControllerImpl.prototype.reset = function () {
            this.mWaitForRelations = {};
            this.mSimultaneousRelations = {};
        };
        return GestureHandlerInteractionControllerImpl;
    }(com.swmansion.gesturehandler.GestureHandlerInteractionController));
    exports.GestureHandlerInteractionController = GestureHandlerInteractionControllerImpl;
}
var Manager = /** @class */ (function (_super) {
    __extends(Manager, _super);
    function Manager() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.viewListeners = new Map();
        return _this;
    }
    Manager.prototype.configureInteractions = function (handler, options) {
        this.interactionManager.configureInteractions(handler.getNative(), options);
    };
    Object.defineProperty(Manager.prototype, "interactionManager", {
        get: function () {
            if (!this._interactionManager) {
                initGestureHandlerInteractionController();
                this._interactionManager = new exports.GestureHandlerInteractionController();
            }
            return this._interactionManager;
        },
        enumerable: true,
        configurable: true
    });
    Manager.getInstance = function () {
        if (!Manager.sManager) {
            Manager.sManager = new Manager();
        }
        return Manager.sManager;
    };
    Manager.prototype.createGestureHandler = function (handlerName, handlerTag, config) {
        var handler = null;
        switch (handlerName) {
            case 'tap':
                handler = new TapGestureHandler(config);
                break;
            case 'pan':
                handler = new PanGestureHandler(config);
                break;
            case 'nativeView':
                handler = new NativeViewGestureHandler(config);
                break;
            case 'pinch':
                handler = new PinchGestureHandler(config);
                break;
            case 'fling':
                handler = new FlingGestureHandler(config);
                break;
            case 'rotation':
                handler = new RotationGestureHandler(config);
                break;
            case 'longPress':
                handler = new LongPressGestureHandler(config);
                break;
        }
        if (handler) {
            handler.manager = new WeakRef(this);
            handler.setTag(handlerTag);
        }
        return handler;
    };
    Manager.prototype.attachGestureHandlerToView = function (handler, view) {
        var nHandler = handler.getNative();
        if (nHandler) {
            var registry = view.page.registry;
            if (registry) {
                registry.registerHandler(nHandler);
                registry.attachHandlerToView(nHandler.getTag(), view.nativeView);
            }
        }
    };
    Manager.prototype.detachGestureHandlerFromView = function (handler, view) {
        var nHandler = handler.getNative();
        if (nHandler) {
            var registry = view.page.registry;
            if (registry) {
                registry.dropHandler(handler.getNative());
            }
        }
    };
    Manager.prototype.attachGestureHandler = function (handler, view) {
        var _this = this;
        if (view.nativeView) {
            this.attachGestureHandlerToView(handler, view);
        }
        var onInit = function () { return _this.attachGestureHandlerToView(handler, view); };
        var onDispose = function () { return _this.detachGestureHandlerFromView(handler, view); };
        view.on(gesturehandler_common_1.ViewInitEvent, onInit, this);
        view.on(gesturehandler_common_1.ViewDisposeEvent, onDispose, this);
        var viewListeners = this.viewListeners.get(view);
        if (!viewListeners) {
            viewListeners = new Map();
            this.viewListeners.set(view, viewListeners);
        }
        viewListeners.set(handler.getTag(), {
            init: onInit,
            dispose: onDispose
        });
    };
    Manager.prototype.detachGestureHandler = function (handler, view) {
        if (view) {
            var viewListeners = this.viewListeners.get(view);
            if (viewListeners) {
                var listeners_1 = viewListeners.get(handler.getTag());
                if (listeners_1) {
                    view.off(gesturehandler_common_1.ViewInitEvent, listeners_1.init, this);
                    view.off(gesturehandler_common_1.ViewDisposeEvent, function () { return listeners_1.dispose; }, this);
                    viewListeners.delete(handler.getTag());
                    if (viewListeners.size === 0) {
                        this.viewListeners.delete(view);
                    }
                }
            }
        }
        this.detachGestureHandlerFromView(handler, view);
    };
    return Manager;
}(gesturehandler_common_1.Manager));
exports.Manager = Manager;
//# sourceMappingURL=gesturehandler.android.js.map