"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var gesturehandler_common_1 = require("./gesturehandler.common");
exports.GestureHandlerStateEvent = gesturehandler_common_1.GestureHandlerStateEvent;
exports.GestureHandlerTouchEvent = gesturehandler_common_1.GestureHandlerTouchEvent;
exports.GestureState = gesturehandler_common_1.GestureState;
exports.HandlerType = gesturehandler_common_1.HandlerType;
exports.install = gesturehandler_common_1.install;
exports.ViewDisposeEvent = gesturehandler_common_1.ViewDisposeEvent;
exports.ViewInitEvent = gesturehandler_common_1.ViewInitEvent;
var types_1 = require("tns-core-modules/utils/types");
var gesturehandler_1 = require("./gesturehandler");
function toJsObject(objCObj) {
    if (objCObj === null || typeof objCObj !== 'object') {
        return objCObj;
    }
    var node, key, i, l;
    var oKeyArr = objCObj.allKeys;
    if (oKeyArr === undefined && objCObj.count !== undefined) {
        // array
        node = [];
        for (i = 0, l = objCObj.count; i < l; i++) {
            key = objCObj.objectAtIndex(i);
            node.push(toJsObject(key));
        }
    }
    else if (oKeyArr !== undefined) {
        // object
        node = {};
        for (i = 0, l = oKeyArr.count; i < l; i++) {
            key = oKeyArr.objectAtIndex(i);
            var val = objCObj.valueForKey(key);
            // Firestore can store nulls
            if (val === null) {
                node[key] = null;
                continue;
            }
            node[key] = getValueForClass(val);
        }
    }
    else {
        node = getValueForClass(objCObj);
    }
    return node;
}
function getValueForClass(val) {
    switch (types_1.getClass(val)) {
        case 'NSArray':
        case 'NSMutableArray':
            return this.toJsObject(val);
        case 'NSDictionary':
        case 'NSMutableDictionary':
            return this.toJsObject(val);
        case 'String':
            return String(val);
        case 'Boolean':
            return val;
        case 'Number':
        case 'NSDecimalNumber':
            return Number(String(val));
        case 'Date':
            return new Date(val);
        default:
            console.log("Please report this at https://github.com/EddyVerbruggen/nativescript-plugin-firebase/issues: iOS toJsObject is missing a converter for class '" +
                types_1.getClass(val) +
                "'. Casting to String as a fallback.");
            return String(val);
    }
}
var HandlerDelegate = /** @class */ (function (_super) {
    __extends(HandlerDelegate, _super);
    function HandlerDelegate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HandlerDelegate.new = function () {
        return _super.new.call(this);
    };
    HandlerDelegate.prototype.initWithOwner = function (owner) {
        this._owner = owner;
        return this;
    };
    HandlerDelegate.prototype.gestureHandlerDidChangeStatePrevStateExtraDataView = function (handler, state, prevState, extraData, view) {
        var owner = this._owner && this._owner.get();
        if (owner) {
            owner.notify({
                eventName: gesturehandler_common_1.GestureHandlerStateEvent,
                object: owner,
                data: {
                    state: state,
                    prevState: prevState,
                    ios: view,
                    extraData: toJsObject(extraData.data),
                    view: view.nsView ? view.nsView.get() : null
                }
            });
        }
    };
    HandlerDelegate.prototype.gestureHandlerTouchEventOnViewStateExtraData = function (handler, view, state, extraData) {
        var owner = this._owner && this._owner.get();
        if (owner) {
            owner.notify({
                eventName: gesturehandler_common_1.GestureHandlerTouchEvent,
                object: owner,
                data: {
                    state: state,
                    ios: view,
                    extraData: toJsObject(extraData.data),
                    view: view.nsView ? view.nsView.get() : null
                }
            });
        }
    };
    HandlerDelegate.ObjCProtocols = [GestureHandlerDelegate];
    return HandlerDelegate;
}(NSObject));
exports.HandlerDelegate = HandlerDelegate;
var Handler = /** @class */ (function (_super) {
    __extends(Handler, _super);
    function Handler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Handler.prototype.createNative = function () {
        return null;
    };
    Handler.prototype.attachToView = function (view) {
        var tag = this.native.tag;
        this.delegate = HandlerDelegate.new().initWithOwner(new WeakRef(this));
        this.native.delegate = this.delegate;
        this.manager.get().attachGestureHandler(tag, view);
    };
    Handler.prototype.detachFromView = function (view) {
        var tag = this.native.tag;
        this.delegate = this.native.delegate = null;
        this.manager.get().attachGestureHandler(tag, view);
    };
    Handler.prototype.getTag = function () {
        return this.native.tag;
    };
    Handler.prototype.setTag = function (value) {
        // cant change tag on ios
    };
    Handler.prototype.getView = function () {
        return null;
    };
    Handler.prototype.cancel = function () {
        this.native.reset();
    };
    __decorate([
        gesturehandler_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "enabled", void 0);
    __decorate([
        gesturehandler_1.nativeProperty,
        __metadata("design:type", Boolean)
    ], Handler.prototype, "shouldCancelWhenOutside", void 0);
    return Handler;
}(gesturehandler_common_1.BaseNative));
exports.Handler = Handler;
var Manager = /** @class */ (function (_super) {
    __extends(Manager, _super);
    function Manager() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.viewListeners = new Map();
        return _this;
    }
    Object.defineProperty(Manager.prototype, "manager", {
        get: function () {
            if (!this._manager) {
                this._manager = GestureHandlerManager.alloc().init();
            }
            return this._manager;
        },
        enumerable: true,
        configurable: true
    });
    Manager.getInstance = function () {
        if (!Manager.sManager) {
            Manager.sManager = new Manager();
        }
        return Manager.sManager;
    };
    Manager.prototype.createGestureHandler = function (handlerName, handlerTag, config) {
        var nHandler = this.manager.createGestureHandlerTagConfig(handlerName, handlerTag, config ? NSDictionary.dictionaryWithDictionary(config) : null);
        var result = new Handler(config, nHandler);
        result.manager = new WeakRef(this);
        return result;
    };
    Manager.prototype.attachGestureHandler = function (handlerTag, view) {
        var _this = this;
        if (view.nativeView) {
            this.manager.attachGestureHandlerToView(handlerTag, view.nativeView);
        }
        var onInit = function () {
            // console.log('attachGestureHandlerToView', handlerTag, view.nativeView);
            _this.manager.attachGestureHandlerToView(handlerTag, view.nativeView);
        };
        var onDispose = function () { return _this.manager.dropGestureHandler(handlerTag); };
        view.on(gesturehandler_common_1.ViewInitEvent, onInit, this);
        view.on(gesturehandler_common_1.ViewDisposeEvent, onDispose, this);
        var viewListeners = this.viewListeners.get(view);
        if (!viewListeners) {
            viewListeners = new Map();
            this.viewListeners.set(view, viewListeners);
        }
        viewListeners.set(handlerTag, {
            init: onInit,
            dispose: onDispose
        });
    };
    Manager.prototype.detachGestureHandler = function (handlerTag, view) {
        if (view.nativeView) {
            this.manager.attachGestureHandlerToView(handlerTag, view.nativeView);
        }
        if (view) {
            var viewListeners = this.viewListeners.get(view);
            if (viewListeners) {
                var listeners_1 = viewListeners.get(handlerTag);
                if (listeners_1) {
                    view.off(gesturehandler_common_1.ViewInitEvent, listeners_1.init, this);
                    view.off(gesturehandler_common_1.ViewDisposeEvent, function () { return listeners_1.dispose; }, this);
                    viewListeners.delete(handlerTag);
                    if (viewListeners.size === 0) {
                        this.viewListeners.delete(view);
                    }
                }
            }
        }
    };
    return Manager;
}(gesturehandler_common_1.Manager));
exports.Manager = Manager;
//# sourceMappingURL=gesturehandler.ios.js.map